from collections import OrderedDict

from django.apps import apps
from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ObjectDoesNotExist

from rest_framework import serializers


class DisplayChoiceField(serializers.ChoiceField):
    """A custom serializer field to use for a choice field."""

    def __init__(self, *args, **kwargs):
        choices = kwargs.get('choices')
        self._choices = OrderedDict(choices)
        super(DisplayChoiceField, self).__init__(*args, **kwargs)

    def to_representation(self, value):
        """Convert a choice internal value to the representation value

        Args:
            value (int): Internal value

        Returns:
            str: Representation value
        """
        try:
            return self._choices[value]
        except KeyError:
            raise Exception('Value: {0} not valid!'.format(value))

    def to_internal_value(self, value):
        """Convert a choice representation value to an internal value

        Args:
            value (str): Representation value

        Returns:
            int: Internal value
        """
        for key, key_value in self._choices.items():
            if key_value.strip().lower() == value.strip().lower():
                return key

        raise Exception('Value: {0} not supported!'.format(value))


class UserRelatedField(serializers.RelatedField):
    """A custom serializer field to use for the `user` relationship."""

    def to_representation(self, value):
        """Convert a User instance to representation (model)

        Args:
            value: User model instance

        Returns:
            str: username
        """
        model = apps.get_model(getattr(settings, 'AUTH_USER_MODEL'))
        try:
            return str(model.objects.get(id=value.id))
        except ObjectDoesNotExist:
            raise Exception('User Id: {0} not found!'.format(value.id))

    def to_internal_value(self, value):
        """Convert a username to a User model instance

        Args:
            value (str): username

        Returns:
            User: User model instance
        """
        model = apps.get_model(getattr(settings, 'AUTH_USER_MODEL'))
        try:
            return model.objects.get(username=value)
        except ObjectDoesNotExist:
            raise Exception('Username: {0} not found!'.format(value))


class ContentRelatedField(serializers.RelatedField):
    """A custom serializer field to use for the `content_type` generic relationship."""

    def to_representation(self, value):
        """Convert a ContentType instance to representation (model)

        Args:
            value: ContentType model instance

        Returns:
            str: model
        """
        try:
            return str(ContentType.objects.get(id=value.id))
        except ObjectDoesNotExist:
            raise Exception('ContentType Id: {0} not found!'.format(value.id))

    def to_internal_value(self, value):
        """Convert a model to a ContentType model instance

        Args:
            value (str): model

        Returns:
            ContentType: ContentType model instance
        """
        try:
            return ContentType.objects.get(model=value.replace(" ", "").lower())
        except ObjectDoesNotExist:
            raise Exception('ContentType Model: {0} not found!'.format(value))



# -*- coding: utf-8 -*-
from collections import OrderedDict

from django.conf import settings
from django_filters import TypedChoiceFilter


class DisplayChoiceFilter(TypedChoiceFilter):

    def __init__(self, *args, **kwargs):
        empty_label = getattr(settings, 'NULL_CHOICE_LABEL', '---------')
        empty_value = getattr(settings, 'NULL_CHOICE_VALUE', None)

        choices = kwargs.pop('choices')
        kwargs['choices'] = [(empty_value, empty_label)] + [(y, y) for x, y in choices]
        kwargs['coerce'] = lambda x: self.coerce(x, choices)

        super(DisplayChoiceFilter, self).__init__(*args, **kwargs)

    @staticmethod
    def coerce(x, choices):
        for key, value in OrderedDict(choices).items():
            if value.lower() == x.lower():
                return key



import gzip
import hashlib

try:
    from StringIO import BytesIO
except ImportError:
    from io import BytesIO

try:
    import urllib2 as urllib
except ImportError:
    from urllib import request as urllib


def calculate_variant_md5(chromosome, position, reference, alternate):
    """Calculate MD5 hash for a variant

    Args:
        chromosome (str): Chromosome
        position (int): Genomic position
        reference (str): Reference allele
        alternate (str): Alternate allele

    Returns:
        str: MD5 hash for a variant
    """
    key = '|'.join(list(map(str, [chromosome, position, reference, alternate])))
    return hashlib.md5(key.encode('utf-8')).hexdigest()


def retrieve_compressed_data(url):
    if url:
        request = urllib.Request(url)
        opener = urllib.build_opener()
        response = opener.open(request)
        gz_data = BytesIO(response.read())
        gzipper = gzip.GzipFile(fileobj=gz_data)
        data = gzipper.read()
        return data.decode('utf-8').strip().split('\n')
    return []


def retrieve_data(url):
    if url:
        return urllib.urlopen(url).read().decode('utf-8').strip().split('\n')
    return []



from django.db import connection
from django.db.models.base import ModelBase
from django.test import TestCase


class ModelMixinTestCase(TestCase):
    """
    Base class for tests of model mixins. To use, subclass and specify
    the mixin class variable. A model using the mixin will be made
    available in self.model.
    """

    def setUp(self):
        """Create a dummy model which extends the mixin"""
        self.model = ModelBase(
            '__TestModel__' + self.mixin.__name__,
            (self.mixin,),
            {'__module__': self.mixin.__module__}
        )

        with connection.schema_editor() as schema_editor:
            schema_editor.create_model(self.model)

    def tearDown(self):
        """Delete the schema for the test model"""
        with connection.schema_editor() as schema_editor:
            schema_editor.delete_model(self.model)
